@@ -37,6 +37,10 @@ namespace Beagle.Filters {
 		static Regex package_pattern = new Regex ("(?<name>([^0-9]+))-(?<version>(.+)).ebuild");
 
 		public FilterEbuild () 
+		{
+		}
+
+		protected override void RegisterSupportedTypes ()
 		{
 			AddSupportedFlavor (FilterFlavor.NewFromExtension (".ebuild"));
 		}
@@ -102,7 +106,7 @@ namespace Beagle.Filters {
 							desktop_file = installed;
 					}
 				}
-				catch (IOException e) {
+				catch (IOException) {
 					// Mostly likely caused by permission error
 				}
 
