@@ -37,9 +37,15 @@ using Beagle.Util;
 
 namespace Beagle.Filters {
 
+	[PropertyKeywordMapping (Keyword="mailfrom",     PropertyName="fixme:from_name",    IsKeyword=false)]
+	[PropertyKeywordMapping (Keyword="mailfromaddr", PropertyName="fixme:from_address", IsKeyword=false)]
+	[PropertyKeywordMapping (Keyword="mailto",       PropertyName="fixme:to_name",      IsKeyword=false)]
+	[PropertyKeywordMapping (Keyword="mailtoaddr",   PropertyName="fixme:to_address",   IsKeyword=false)]
+	[PropertyKeywordMapping (Keyword="mailinglist",  PropertyName="fixme:mlist",        IsKeyword=true, Description="Mailing list id")]
 	public class FilterMail : Beagle.Daemon.Filter, IDisposable {
 
 		private static bool gmime_initialized = false;
+		private static bool gmime_broken = false;
 
 		private GMime.Message message;
 		private PartHandler handler;
@@ -48,23 +54,38 @@ namespace Beagle.Filters {
 		{
 			// 1: Make email addresses non-keyword, add sanitized version
 			//    for eaching for parts of an email address.
-			SetVersion (1);
+			// 2: No need to separately add sanitized version of emails.
+			//    BeagleAnalyzer uses a tokenfilter taking care of this.
+			// 3: Add standard file properties to attachments.
+			SetVersion (3);
+		}
 
+		protected override void RegisterSupportedTypes ()
+		{
 			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("message/rfc822"));
 		}
 
 		protected override void DoOpen (FileInfo info)
 		{
 			if (!gmime_initialized) {
+				gmime_initialized = true;
+
+				// Only try initialization once; if it fails, it'll never work.
 				try {
 					GMime.Global.Init ();
-					gmime_initialized = true;
-				} catch {
+				} catch (Exception e) {
+					Log.Error (e, "Unable to initialize GMime");
+					gmime_broken = true;
 					Error ();
 					return;
 				}
 			}
 
+			if (gmime_broken) {
+				Error ();
+				return;
+			}
+
 			int mail_fd = Mono.Unix.Native.Syscall.open (info.FullName, Mono.Unix.Native.OpenFlags.O_RDONLY);
 			
 			if (mail_fd == -1)
@@ -75,6 +96,9 @@ namespace Beagle.Filters {
 			this.message = parser.ConstructMessage ();
 			stream.Dispose ();
 			parser.Dispose ();
+
+			if (this.message == null)
+				Error ();
 		}
 
 		private bool HasAttachments (GMime.Object mime_part)
@@ -100,27 +124,30 @@ namespace Beagle.Filters {
 			GMime.InternetAddressList addrs;
 			addrs = this.message.GetRecipients (GMime.Message.RecipientType.To);
 			foreach (GMime.InternetAddress ia in addrs) {
-				AddProperty (Property.NewKeyword ("fixme:to", ia.ToString (false)));
-				AddProperty (Property.New ("fixme:to_address", ia.Addr));
-				AddProperty (Property.NewUnstored ("fixme:to_sanitized", StringFu.SanitizeEmail (ia.Addr)));
+				AddProperty (Property.NewUnsearched ("fixme:to", ia.ToString (false)));
+				if (ia.AddressType != GMime.InternetAddressType.Group)
+					AddProperty (Property.New ("fixme:to_address", ia.Addr));
+
 				AddProperty (Property.New ("fixme:to_name", ia.Name));
 			}
 			addrs.Dispose ();
 
 			addrs = this.message.GetRecipients (GMime.Message.RecipientType.Cc);
 			foreach (GMime.InternetAddress ia in addrs) {
-				AddProperty (Property.NewKeyword ("fixme:cc", ia.ToString (false)));
-				AddProperty (Property.New ("fixme:cc_address", ia.Addr));
-				AddProperty (Property.NewUnstored ("fixme:cc_sanitized", StringFu.SanitizeEmail (ia.Addr)));
+				AddProperty (Property.NewUnsearched ("fixme:cc", ia.ToString (false)));
+				if (ia.AddressType != GMime.InternetAddressType.Group)
+					AddProperty (Property.New ("fixme:cc_address", ia.Addr));
+
 				AddProperty (Property.New ("fixme:cc_name", ia.Name));
 			}
 			addrs.Dispose ();
 
 			addrs = GMime.InternetAddressList.ParseString (GMime.Utils.HeaderDecodePhrase (this.message.Sender));
 			foreach (GMime.InternetAddress ia in addrs) {
-				AddProperty (Property.NewKeyword ("fixme:from", ia.ToString (false)));
-				AddProperty (Property.New ("fixme:from_address", ia.Addr));
-				AddProperty (Property.NewUnstored ("fixme:from_sanitized", StringFu.SanitizeEmail (ia.Addr)));
+				AddProperty (Property.NewUnsearched ("fixme:from", ia.ToString (false)));
+				if (ia.AddressType != GMime.InternetAddressType.Group)
+					AddProperty (Property.New ("fixme:from_address", ia.Addr));
+
 				AddProperty (Property.New ("fixme:from_name", ia.Name));
 			}
 			addrs.Dispose ();
@@ -143,6 +170,20 @@ namespace Beagle.Filters {
 				// FIXME: Might need some additional parsing.
 				AddProperty (Property.NewKeyword ("fixme:mlist", GMime.Utils.HeaderDecodePhrase (list_id)));
 			}
+
+			// KMail can store replies in the same folder
+			// Use issent flag to distinguish between incoming
+			// and outgoing message
+			string kmail_msg_sent = this.message.GetHeader ("X-KMail-Link-Type");
+			bool issent_is_set = false;
+			foreach (Property property in IndexableProperties) {
+				if (property.Key == "fixme:isSent") {
+					issent_is_set = true;
+					break;
+				}
+			}
+			if (!issent_is_set && kmail_msg_sent != null && kmail_msg_sent == "reply")
+				AddProperty (Property.NewFlag ("fixme:isSent"));
 		}
 
 		protected override void DoPullSetup ()
@@ -163,10 +204,12 @@ namespace Beagle.Filters {
 
 			string l = handler.Reader.ReadLine ();
 
-			if (l != null)
-				AppendText (l);
-			else
+			if (l == null)
 				Finished ();
+			else if (l.Length > 0) {
+				AppendText (l);
+				AppendStructuralBreak ();
+			}
 		}
 
 		protected override void DoClose ()
@@ -193,6 +236,17 @@ namespace Beagle.Filters {
 			private ArrayList child_indexables = new ArrayList ();
 			private TextReader reader;
 
+			// Blacklist a handful of common MIME types that are
+			// either pointless on their own or ones that we don't
+			// have filters for.
+			static private string[] blacklisted_mime_types = new string[] {
+				"application/pgp-signature",
+				"application/x-pkcs7-signature",
+				"application/ms-tnef",
+				"text/x-vcalendar",
+				"text/x-vcard"
+			};
+
 			public PartHandler (Beagle.Daemon.Filter filter)
 			{
 				this.filter = filter;
@@ -273,8 +327,10 @@ namespace Beagle.Filters {
 					// creating a child indexable for it.
 					bool no_child_needed = false;
 
+					string mime_type = part.ContentType.ToString ().ToLower ();
+
 					if (this.depth == 1 && this.count == 0) {
-						if (part.ContentType.ToString ().ToLower () == "text/plain") {
+						if (mime_type == "text/plain") {
 							no_child_needed = true;
 
 							this.reader = new StreamReader (stream);
@@ -282,21 +338,40 @@ namespace Beagle.Filters {
 					}
 
 					if (!no_child_needed) {
-						string sub_uri = this.filter.Uri.ToString () + "#" + this.count;
-						Indexable child = new Indexable (new Uri (sub_uri));
+						// Check the mime type against the blacklist and don't index any
+						// parts that are contained within.  That way the user doesn't
+						// get flooded with pointless signatures and vcard and ical
+						// attachments along with (real) attachments.
 
-						child.HitType = "MailMessage";
-						child.MimeType = part.ContentType.ToString ();
-						child.CacheContent = false;
+						if (Array.IndexOf (blacklisted_mime_types, mime_type) == -1) {
+							string sub_uri = this.filter.Uri.ToString () + "#" + this.count;
+							Indexable child = new Indexable (new Uri (sub_uri));
 
-						child.AddProperty (Property.NewKeyword ("fixme:attachment_title", ((GMime.Part)part).Filename));
+							child.DisplayUri = new Uri (this.filter.DisplayUri.ToString () + "#" + this.count);
 
-						if (part.ContentType.Type.ToLower () == "text")
-							child.SetTextReader (new StreamReader (stream));
-						else
-							child.SetBinaryStream (stream);
+							child.HitType = "MailMessage";
+							child.MimeType = mime_type;
+							child.CacheContent = false;
 
-						this.child_indexables.Add (child);
+							string filename = ((GMime.Part) part).Filename;
+
+							if (! String.IsNullOrEmpty (filename)) {
+								child.AddProperty (Property.NewKeyword ("fixme:attachment_title", filename));
+
+								foreach (Property prop in Property.StandardFileProperties (filename, false))
+									child.AddProperty (prop);
+							}
+
+							if (part.ContentType.Type.ToLower () == "text")
+								child.SetTextReader (new StreamReader (stream));
+							else
+								child.SetBinaryStream (stream);
+
+							this.child_indexables.Add (child);
+						} else {
+							Log.Debug ("Skipping attachment {0}#{1} with blacklisted mime type {2}",
+								   this.filter.Uri, this.count, mime_type);
+						}
 					}
 
 					this.count++;
