@@ -177,27 +177,35 @@ namespace Beagle.Daemon {
 			if (indexable.ContentUri.IsFile) {
 				path = indexable.ContentUri.LocalPath;
 
-				// Otherwise sniff the mime-type from the file
-				if (indexable.MimeType == null)
-					indexable.MimeType = XdgMime.GetMimeType (path);
+				// Otherwise, set the mime type for a directory,
+				// or sniff it from the file.
+				if (indexable.MimeType == null) {
+					if (Directory.Exists (path)) {
+						indexable.MimeType = "inode/directory";
+						indexable.NoContent = true;
+					} else if (File.Exists (path)) {
+						indexable.MimeType = XdgMime.GetMimeType (path);
+					} else {
+						Log.Warn ("Unable to filter {0}.  {1} not found.", indexable.DisplayUri, path);
+						return false;
+					}
+				}
+
+				// Set the timestamp to the last write time, if it isn't
+				// set by the backend.
+				if (! indexable.ValidTimestamp && indexable.IsNonTransient)
+					indexable.Timestamp = FileSystem.GetLastWriteTimeUtc (path);
+
+				// Check the timestamp to make sure the file hasn't
+				// disappeared from underneath us.
+				if (! FileSystem.ExistsByDateTime (indexable.Timestamp)) {
+					Log.Warn ("Unable to filter {0}.  {1} appears to have disappeared from underneath us", indexable.DisplayUri, path);
+					return false;
+				}
 
 				if (filters == null || filters.Count == 0) {
 					filters = CreateFiltersFromIndexable (indexable);
 				}
-
-				if (Directory.Exists (path)) {
-					indexable.MimeType = "inode/directory";
-					indexable.NoContent = true;
-					if (! indexable.ValidTimestamp && indexable.IsNonTransient)
-						indexable.Timestamp = Directory.GetLastWriteTimeUtc (path);
-				} else if (File.Exists (path)) {
-					// Set the timestamp to the best possible estimate (if no timestamp was set by the backend)
-					if (! indexable.ValidTimestamp && indexable.IsNonTransient)
-						indexable.Timestamp = File.GetLastWriteTimeUtc (path);
-				} else {
-					Logger.Log.Warn ("No such file: {0}", path);
-					return false;
-				}
 			}
 
 			// We don't know how to filter this, so there is nothing else to do.
@@ -263,8 +271,8 @@ namespace Beagle.Daemon {
 
 					filter = candidate_filter;
 					return true;
-				} else if (Debug) {
-					Logger.Log.Debug ("Unsuccessfully filtered {0} with {1}, falling back", path, candidate_filter);
+				} else {
+					Log.Warn ("Error in filtering {0} with {1}, falling back", path, candidate_filter);
 					candidate_filter.Cleanup ();
 				}
 			}
