@@ -109,7 +109,7 @@ namespace Beagle.Daemon.KMailQueryable {
 			// - if is in one of the mail_directories, index it if it is an mbox file
 			if ((type & Inotify.EventType.Create) != 0 && (type & Inotify.EventType.IsDirectory) == 0) {
 				if (IsMailDir (path)) {
-					Indexable indexable = MaildirMessageToIndexable (fullPath);
+					Indexable indexable = MaildirMessageToIndexable (fullPath, false);
 					AddIndexableTask (indexable, fullPath);
 				} else {
 					// add mbox file to mbox_files
@@ -141,7 +141,7 @@ namespace Beagle.Daemon.KMailQueryable {
 			// - need to delete from the source
 			if ((type & Inotify.EventType.MovedTo) != 0 && (type & Inotify.EventType.IsDirectory) == 0) {
 				if (IsMailDir (path)) {
-					Indexable indexable = MaildirMessageToIndexable (fullPath);
+					Indexable indexable = MaildirMessageToIndexable (fullPath, false);
 					AddIndexableTask (indexable, fullPath);
 				}
 				if (IsMailDir (srcpath))
@@ -359,7 +359,7 @@ namespace Beagle.Daemon.KMailQueryable {
 		/** 
 		 * Create an indexable from a maildir message
 		 */
-		public Indexable MaildirMessageToIndexable (string filename)
+		public Indexable MaildirMessageToIndexable (string filename, bool crawl)
 		{
 			//Logger.Log.Debug ("+ indexing maildir mail:" + filename);
 			String folder = GetFolderMaildir(filename);
@@ -369,6 +369,7 @@ namespace Beagle.Daemon.KMailQueryable {
 			indexable.HitType = "MailMessage";
 			indexable.MimeType = "message/rfc822";
 			indexable.CacheContent = false;
+			indexable.Crawled = crawl;
 
 			indexable.AddProperty (Property.NewUnsearched ("fixme:client", "kmail"));
 			indexable.AddProperty (Property.NewUnsearched ("fixme:account", account_name));
@@ -394,6 +395,7 @@ namespace Beagle.Daemon.KMailQueryable {
 			indexable.HitType = "MailMessage";
 			indexable.MimeType = "message/rfc822";
 			indexable.CacheContent = false;
+			indexable.Crawled = true; // mbox files will be stored in temp files, so flush them when done
 
 			indexable.AddProperty (Property.NewUnsearched ("fixme:client", "kmail"));
 			indexable.AddProperty (Property.NewUnsearched ("fixme:account", account_name));
