@@ -40,8 +40,6 @@ namespace Beagle.Daemon.LifereaQueryable {
 	[QueryableFlavor (Name="Liferea", Domain=QueryDomain.Local, RequireInotify=false)]
 	public class LifereaQueryable : LuceneFileQueryable {
 
-		private static Logger log = Logger.Get ("LifereaQueryable");
-
 		string liferea_dir;
 		internal string icon_dir;
 
@@ -61,10 +59,26 @@ namespace Beagle.Daemon.LifereaQueryable {
 		
 		public LifereaQueryable () : base ("LifereaIndex", INDEX_VERSION)
 		{
-			liferea_dir = Path.Combine (PathFinder.HomeDir, ".liferea");
-			liferea_dir = Path.Combine (liferea_dir, "cache");
+		}
+
+		/////////////////////////////////////////////////
+
+		private bool CheckForDirectory ()
+		{
+			string dir = Path.Combine (PathFinder.HomeDir, ".liferea_1.2");
+
+			if (! Directory.Exists (dir))
+				dir = Path.Combine (PathFinder.HomeDir, ".liferea");
+
+			if (! Directory.Exists (dir))
+				return false;
+
+			liferea_dir = Path.Combine (dir, "cache");
 			icon_dir = Path.Combine (liferea_dir, "favicons");
 			liferea_dir = Path.Combine (liferea_dir, "feeds");
+			Log.Debug ("Found Liferea directory '{0}'", liferea_dir);
+
+			return true;
 		}
 
 		/////////////////////////////////////////////////
@@ -78,7 +92,8 @@ namespace Beagle.Daemon.LifereaQueryable {
 
 		private void StartWorker ()
 		{
-			if (!Directory.Exists (liferea_dir)) {
+			if (! CheckForDirectory ()) {
+				Log.Debug ("Watching for creation of Liferea directory");
 				GLib.Timeout.Add (60000, new GLib.TimeoutHandler (CheckForExistence));
                                 return;
 			}
@@ -98,9 +113,8 @@ namespace Beagle.Daemon.LifereaQueryable {
                                 fsw.EnableRaisingEvents = true;
 			}
 
-                        log.Info ("Scanning Liferea feeds...");
+                        Log.Info ("Scanning Liferea feeds...");
 
-			State = QueryableState.Crawling;
 			Stopwatch stopwatch = new Stopwatch ();
 			stopwatch.Start ();
 
@@ -110,17 +124,16 @@ namespace Beagle.Daemon.LifereaQueryable {
 				IndexSingleFeed (file.FullName);
 			}
 
-			State = QueryableState.Idle;
 			stopwatch.Stop ();
-                        log.Info ("{0} files will be parsed (scanned in {1})", count, stopwatch);
+                        Log.Info ("{0} files will be parsed (scanned in {1})", count, stopwatch);
 		}
 
 		private bool CheckForExistence ()
                 {
-                        if (!Directory.Exists (liferea_dir))
+                        if (! CheckForDirectory ())
                                 return true;
 
-                        this.Start ();
+			ExceptionHandlingThread.Start (new ThreadStart (StartWorker));
 
                         return false;
                 }
@@ -157,7 +170,7 @@ namespace Beagle.Daemon.LifereaQueryable {
 
 		private void IndexSingleFeed (string filename) {
 			if (ThisScheduler.ContainsByTag (filename)) {
-				Logger.Log.Debug ("Not adding task for already running task: {0}", filename);
+				Log.Debug ("Not adding task for already running task: {0}", filename);
 				return;
 			}
 
@@ -170,7 +183,7 @@ namespace Beagle.Daemon.LifereaQueryable {
 		}
 
 		private void Removefeed_file (string file) {
-			Logger.Log.Debug ("Removing Liferea feed_file:" + file);
+			Log.Debug ("Removing Liferea feed_file:" + file);
 			Uri uri = UriFu.PathToFileUri (file);
 			Scheduler.Task task = NewRemoveTask (uri);
 			task.Priority = Scheduler.Priority.Immediate;
@@ -233,7 +246,7 @@ namespace Beagle.Daemon.LifereaQueryable {
 				return;
 			}
 			try {
-				Logger.Log.Debug ("Opening liferea feed file: {0}", feed_file);
+				Log.Debug ("Opening liferea feed file: {0}", feed_file);
 				reader = new XmlTextReader (feed_file);
 				reader.WhitespaceHandling = WhitespaceHandling.None;
 				
@@ -266,7 +279,7 @@ namespace Beagle.Daemon.LifereaQueryable {
 					}
 				} while (!reader.EOF && reader.NodeType == XmlNodeType.Element);
 			} catch (XmlException ex) {
-				Logger.Log.Warn (ex, "Caught exception parsing feed file:");
+				Log.Warn (ex, "Caught exception parsing feed file:");
 				is_valid_file = false;
 				reader.Close ();
 			}
@@ -291,8 +304,8 @@ namespace Beagle.Daemon.LifereaQueryable {
 			}
 
 			if (current_item == null) {
-				//Logger.Log.Debug ("LifereaQ: Probably no more feeds left in " + feed_file);
-				//Logger.Log.Debug ("Causing string = " + itemString);
+				//Log.Debug ("LifereaQ: Probably no more feeds left in " + feed_file);
+				//Log.Debug ("Causing string = " + itemString);
 				current_item = null;
 				is_valid_file = false;
 				reader.Close ();
@@ -320,7 +333,7 @@ namespace Beagle.Daemon.LifereaQueryable {
 			indexable.MimeType = "text/html";
 			indexable.HitType = "FeedItem";
 
-			DateTime date = new DateTime (1970, 1, 1);
+			DateTime date = DateTimeUtil.UnixToDateTimeUtc (0);
 			date = date.AddSeconds (current_item.Timestamp);
 			indexable.Timestamp = date;				
 
