@@ -1,5 +1,5 @@
 /*
- * Copyright 2004 The Apache Software Foundation
+ * Copyright 2005 The Apache Software Foundation
  * 
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -13,7 +13,9 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
+
 using System;
+
 namespace Lucene.Net.Search
 {
 	
@@ -93,74 +95,74 @@ namespace Lucene.Net.Search
 		private int end;
 		private Bucket current;
 		
-        public override void  Score(HitCollector hc)
-        {
-            Next();
-            Score(hc, System.Int32.MaxValue);
-        }
+		public override void  Score(HitCollector hc)
+		{
+			Next();
+			Score(hc, System.Int32.MaxValue);
+		}
 		
-        protected internal override bool Score(HitCollector hc, int max)
-        {
-            if (coordFactors == null)
-                ComputeCoordFactors();
+		protected internal override bool Score(HitCollector hc, int max)
+		{
+			if (coordFactors == null)
+				ComputeCoordFactors();
 			
-            bool more;
-            Bucket tmp;
+			bool more;
+			Bucket tmp;
 			
-            do 
-            {
-                bucketTable.first = null;
+			do 
+			{
+				bucketTable.first = null;
 				
-                while (current != null)
-                {
-                    // more queued 
-    					
-                    // check prohibited & required
-                    if ((current.bits & prohibitedMask) == 0 && (current.bits & requiredMask) == requiredMask)
-                    {
-    						
-                        if (current.doc >= max)
-                        {
-                            tmp = current;
-                            current = current.next;
-                            tmp.next = bucketTable.first;
-                            bucketTable.first = tmp;
-                            continue;
-                        }
-    						
-                        hc.Collect(current.doc, current.score * coordFactors[current.coord]);
-                    }
-    					
-                    current = current.next; // pop the queue
-                }
+				while (current != null)
+				{
+					// more queued 
+					
+					// check prohibited & required
+					if ((current.bits & prohibitedMask) == 0 && (current.bits & requiredMask) == requiredMask)
+					{
+						
+						if (current.doc >= max)
+						{
+							tmp = current;
+							current = current.next;
+							tmp.next = bucketTable.first;
+							bucketTable.first = tmp;
+							continue;
+						}
+						
+						hc.Collect(current.doc, current.score * coordFactors[current.coord]);
+					}
+					
+					current = current.next; // pop the queue
+				}
 				
-                if (bucketTable.first != null)
-                {
-                    current = bucketTable.first;
-                    bucketTable.first = current.next;
-                    return true;
-                }
+				if (bucketTable.first != null)
+				{
+					current = bucketTable.first;
+					bucketTable.first = current.next;
+					return true;
+				}
 				
-                // refill the queue
-                more = false;
-                end += BucketTable.SIZE;
-                for (SubScorer sub = scorers; sub != null; sub = sub.next)
-                {
-                    if (!sub.done)
-                    {
-                        sub.done = !sub.scorer.Score(sub.collector, end);
-                        if (!sub.done)
-                            more = true;
-                    }
-                }
-                current = bucketTable.first;
-            }
-            while (current != null || more);
+				// refill the queue
+				more = false;
+				end += BucketTable.SIZE;
+				for (SubScorer sub = scorers; sub != null; sub = sub.next)
+				{
+					if (!sub.done)
+					{
+						sub.done = !sub.scorer.Score(sub.collector, end);
+						if (!sub.done)
+							more = true;
+					}
+				}
+				current = bucketTable.first;
+			}
+			while (current != null || more);
 			
-            return false;
-        }
+			return false;
+		}
 		
-        public override int Doc()
+		public override int Doc()
 		{
 			return current.doc;
 		}
@@ -181,7 +183,7 @@ namespace Lucene.Net.Search
 					{
 						return true;
 					}
-                }
+				}
 				
 				// refill the queue
 				more = false;
@@ -238,6 +240,7 @@ namespace Lucene.Net.Search
 			
 			public BucketTable(BooleanScorer scorer)
 			{
+                InitBlock();
 				this.scorer = scorer;
 			}
 			
