@@ -13,20 +13,24 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
+
 using System;
 using IndexReader = Lucene.Net.Index.IndexReader;
 using Term = Lucene.Net.Index.Term;
 using TermEnum = Lucene.Net.Index.TermEnum;
+using ToStringUtils = Lucene.Net.Util.ToStringUtils;
+
 namespace Lucene.Net.Search
 {
 	
-	/// <summary> A Query that matches documents within an exclusive range.
+	/// <summary> A Query that matches documents within an exclusive range. A RangeQuery
+	/// is built by QueryParser for input like <code>[010 TO 120]</code>.
 	/// 
 	/// </summary>
-	/// <version>  $Id: RangeQuery.cs,v 1.3 2005/10/06 19:29:57 dsd Exp $
+	/// <version>  $Id: RangeQuery.cs,v 1.5 2006/10/02 18:18:01 joeshaw Exp $
 	/// </version>
 	[Serializable]
-	public class RangeQuery:Query
+	public class RangeQuery : Query
 	{
 		private Term lowerTerm;
 		private Term upperTerm;
@@ -36,7 +40,7 @@ namespace Lucene.Net.Search
 		/// <code>lowerTerm</code> but less than <code>upperTerm</code>.
 		/// There must be at least one term and either term may be null,
 		/// in which case there is no bound on that side, but if there are
-		/// two terms, both terms <b>must</b> be for the same Field.
+		/// two terms, both terms <b>must</b> be for the same field.
 		/// </summary>
 		public RangeQuery(Term lowerTerm, Term upperTerm, bool inclusive)
 		{
@@ -44,9 +48,9 @@ namespace Lucene.Net.Search
 			{
 				throw new System.ArgumentException("At least one term must be non-null");
 			}
-			if (lowerTerm != null && upperTerm != null && (System.Object) lowerTerm.Field() != (System.Object) upperTerm.Field())
+			if (lowerTerm != null && upperTerm != null && lowerTerm.Field() != upperTerm.Field())
 			{
-				throw new System.ArgumentException("Both terms must be for the same Field");
+				throw new System.ArgumentException("Both terms must be for the same field");
 			}
 			
 			// if we have a lowerTerm, start there. otherwise, start at beginning
@@ -82,7 +86,7 @@ namespace Lucene.Net.Search
 				do 
 				{
 					Term term = enumerator.Term();
-					if (term != null && (System.Object) term.Field() == (System.Object) testField)
+					if (term != null && term.Field() == testField)
 					{
 						if (!checkLower || String.CompareOrdinal(term.Text(), lowerTerm.Text()) > 0)
 						{
@@ -114,12 +118,7 @@ namespace Lucene.Net.Search
 			return query;
 		}
 		
-		public override Query Combine(Query[] queries)
-		{
-			return Query.MergeBooleanQueries(queries);
-		}
-		
-		/// <summary>Returns the Field name for this query </summary>
+		/// <summary>Returns the field name for this query </summary>
 		public virtual System.String GetField()
 		{
 			return (lowerTerm != null?lowerTerm.Field():upperTerm.Field());
@@ -154,46 +153,39 @@ namespace Lucene.Net.Search
 				buffer.Append(":");
 			}
 			buffer.Append(inclusive?"[":"{");
-			buffer.Append(lowerTerm != null?lowerTerm.Text():"null");
+			buffer.Append(lowerTerm != null ? lowerTerm.Text() : "null");
 			buffer.Append(" TO ");
-			buffer.Append(upperTerm != null?upperTerm.Text():"null");
-			buffer.Append(inclusive?"]":"}");
-			if (GetBoost() != 1.0f)
-			{
-                System.Globalization.NumberFormatInfo nfi = new System.Globalization.CultureInfo("en-US", false).NumberFormat;
-                nfi.NumberDecimalDigits = 1;
-
-				buffer.Append("^");
-				buffer.Append(GetBoost().ToString("N", nfi));
-			}
+			buffer.Append(upperTerm != null ? upperTerm.Text() : "null");
+			buffer.Append(inclusive ? "]" : "}");
+			buffer.Append(ToStringUtils.Boost(GetBoost()));
 			return buffer.ToString();
 		}
 		
-        /// <summary>Returns true iff <code>o</code> is equal to this. </summary>
-        public  override bool Equals(System.Object o)
-        {
-            if (this == o)
-                return true;
-            if (!(o is RangeQuery))
-                return false;
+		/// <summary>Returns true iff <code>o</code> is equal to this. </summary>
+		public  override bool Equals(System.Object o)
+		{
+			if (this == o)
+				return true;
+			if (!(o is RangeQuery))
+				return false;
 			
-            RangeQuery other = (RangeQuery) o;
-            if (this.GetBoost() != other.GetBoost())
-                return false;
-            if (this.inclusive != other.inclusive)
-                return false;
-            // one of lowerTerm and upperTerm can be null
-            if (this.lowerTerm != null ? !this.lowerTerm.Equals(other.lowerTerm) : other.lowerTerm != null)
-                return false;
-            if (this.upperTerm != null ? !this.upperTerm.Equals(other.upperTerm) : other.upperTerm != null)
-                return false;
-            return true;
-        }
+			RangeQuery other = (RangeQuery) o;
+			if (this.GetBoost() != other.GetBoost())
+				return false;
+			if (this.inclusive != other.inclusive)
+				return false;
+			// one of lowerTerm and upperTerm can be null
+			if (this.lowerTerm != null?!this.lowerTerm.Equals(other.lowerTerm):other.lowerTerm != null)
+				return false;
+			if (this.upperTerm != null?!this.upperTerm.Equals(other.upperTerm):other.upperTerm != null)
+				return false;
+			return true;
+		}
 		
-        /// <summary>Returns a hash code value for this object.</summary>
-        public override int GetHashCode()
-        {
-            return BitConverter.ToInt32(BitConverter.GetBytes(GetBoost()), 0) ^ (lowerTerm != null ? lowerTerm.GetHashCode() : 0) ^ (upperTerm != null ? upperTerm.GetHashCode() : 0) ^ (this.inclusive ? 1 : 0);
-        }
-    }
+		/// <summary>Returns a hash code value for this object.</summary>
+		public override int GetHashCode()
+		{
+			return BitConverter.ToInt32(BitConverter.GetBytes(GetBoost()), 0) ^ (lowerTerm != null ? lowerTerm.GetHashCode():0) ^ (upperTerm != null?upperTerm.GetHashCode() : 0) ^ (this.inclusive ? 1 : 0);
+		}
+	}
 }
\ No newline at end of file
