@@ -13,17 +13,24 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
+
 using System;
 using Document = Lucene.Net.Documents.Document;
 using IndexReader = Lucene.Net.Index.IndexReader;
 using Term = Lucene.Net.Index.Term;
+
 namespace Lucene.Net.Search
 {
 	
 	/// <summary>The interface for search implementations.
 	/// 
-	/// <p>Implementations provide search over a single index, over multiple
+	/// <p>Searchable is the abstract network protocol for searching. 
+	/// Implementations provide search over a single index, over multiple
 	/// indices, and over indices on remote servers.
+	/// 
+	/// <p>Queries, filters and sort criteria are designed to be compact so that
+	/// they may be efficiently passed to a remote index, with only the top-scoring
+	/// hits being returned, rather than every non-zero scoring hit.
 	/// </summary>
 	
     public interface Searchable
@@ -32,6 +39,7 @@ namespace Lucene.Net.Search
         /// 
         /// <p>{@link HitCollector#Collect(int,float)} is called for every non-zero
         /// scoring document.
+        /// <br>HitCollector-based access to remote indexes is discouraged.
         /// 
         /// <p>Applications should only use this if they need <i>all</i> of the
         /// matching documents.  The high-level search API ({@link
@@ -45,8 +53,19 @@ namespace Lucene.Net.Search
         /// </param>
         /// <param name="results">to receive hits
         /// </param>
+        /// <throws>  BooleanQuery.TooManyClauses </throws>
+        /// <summary> 
+        /// </summary>
+        /// <deprecated>
+        /// </deprecated>
         void  Search(Query query, Filter filter, HitCollector results);
-		
+
+        /// <summary>Expert: Low-level search implementation.
+        /// Identical to {@link #Search(Query, Filter, HitCollector)}, but takes
+        /// a Weight instead of a query.
+        /// </summary>
+        void  Search(Weight weight, Filter filter, HitCollector results);
+
         /// <summary>Frees resources associated with this Searcher.
         /// Be careful not to call this method while you are still using objects
         /// like {@link Hits}.
@@ -56,14 +75,20 @@ namespace Lucene.Net.Search
         /// <summary>Expert: Returns the number of documents containing <code>term</code>.
         /// Called by search code to compute term weights.
         /// </summary>
-        /// <seealso cref="IndexReader#docFreq(Term).">
+        /// <seealso cref="IndexReader#docFreq(Term)">
         /// </seealso>
         int DocFreq(Term term);
 		
+        /// <summary>Expert: For each term in the terms array, calculates the number of
+        /// documents containing <code>term</code>. Returns an array with these
+        /// document frequencies. Used to minimize number of remote calls.
+        /// </summary>
+        int[] DocFreqs(Term[] terms);
+		
         /// <summary>Expert: Returns one greater than the largest possible document number.
         /// Called by search code to compute term weights.
         /// </summary>
-        /// <seealso cref="IndexReader#maxDoc().">
+        /// <seealso cref="IndexReader#maxDoc()">
         /// </seealso>
         int MaxDoc();
 		
@@ -75,16 +100,28 @@ namespace Lucene.Net.Search
         /// <p>Applications should usually call {@link Searcher#Search(Query)} or
         /// {@link Searcher#Search(Query,Filter)} instead.
         /// </summary>
+        /// <throws>  BooleanQuery.TooManyClauses </throws>
+        /// <summary> 
+        /// </summary>
+        /// <deprecated>
+        /// </deprecated>
         TopDocs Search(Query query, Filter filter, int n);
 		
+        /// <summary>Expert: Low-level search implementation.
+        /// Identical to {@link #Search(Query, Filter, int)}, but takes
+        /// a Weight instead of a query.
+        /// </summary>
+        TopDocs Search(Weight weight, Filter filter, int n);
+		
         /// <summary>Expert: Returns the stored fields of document <code>i</code>.
         /// Called by {@link HitCollector} implementations.
         /// </summary>
-        /// <seealso cref="IndexReader#document(int).">
+        /// <seealso cref="IndexReader#document(int)">
         /// </seealso>
         Document Doc(int i);
 		
-        /// <summary>Expert: called to re-write queries into primitive queries. </summary>
+        /// <summary>Expert: called to re-write queries into primitive queries.</summary>
+        /// <throws>  BooleanQuery.TooManyClauses </throws>
         Query Rewrite(Query query);
 		
         /// <summary>Returns an Explanation that describes how <code>doc</code> scored against
@@ -95,8 +132,14 @@ namespace Lucene.Net.Search
         /// Computing an explanation is as expensive as executing the query over the
         /// entire index.
         /// </summary>
+        /// <throws>  BooleanQuery.TooManyClauses </throws>
         Explanation Explain(Query query, int doc);
 		
+        /// <summary> Identical to {@link #Search(Query, Filter, HitCollector)}, but takes
+        /// a Weight instead of a query.
+        /// </summary>
+        Explanation Explain(Weight weight, int doc);
+		
         /// <summary>Expert: Low-level search implementation with arbitrary sorting.  Finds
         /// the top <code>n</code> hits for <code>query</code>, applying
         /// <code>filter</code> if non-null, and sorting the hits by the criteria in
@@ -105,6 +148,17 @@ namespace Lucene.Net.Search
         /// <p>Applications should usually call {@link
         /// Searcher#Search(Query,Filter,Sort)} instead.
         /// </summary>
+        /// <throws>  BooleanQuery.TooManyClauses </throws>
+        /// <summary> 
+        /// </summary>
+        /// <deprecated>
+        /// </deprecated>
         TopFieldDocs Search(Query query, Filter filter, int n, Sort sort);
+		
+        /// <summary>Expert: Low-level search implementation.
+        /// Identical to {@link #Search(Query, Filter, int, Sort)}, but takes
+        /// a Weight instead of a query.
+        /// </summary>
+        TopFieldDocs Search(Weight weight, Filter filter, int n, Sort sort);
     }
 }
\ No newline at end of file
