@@ -14,7 +14,7 @@ namespace Bludgeon {
 		{
 			StringBuilder builder;
 			builder = new StringBuilder ();
-			builder.AppendFormat ("{0:yy-MM-dd HH.mm.ss.ff} ", DateTime.Now);
+			builder.AppendFormat ("{0:yyMMdd HHmmssffff} ", DateTime.Now);
 			builder.Append (prefix);
 			builder.Append (' ');
 			builder.AppendFormat (format, args);
