@@ -3,10 +3,15 @@ using System.Diagnostics;
 using System.Runtime.InteropServices;
 using Mono.Unix;
 
+using GConf;
+
 namespace Search.Tiles {
 
 	public class ApplicationActivator : TileActivator {
 
+		static bool checked_gconf = false;
+		static bool disable_command_line = false;
+
 		public ApplicationActivator () : base ()
 		{
 			AddSupportedFlavor (new HitFlavor (null, null, "application/x-desktop"));
@@ -29,6 +34,20 @@ namespace Search.Tiles {
 				gnome_desktop_item_unref (ditem);
 		}
 
+		static void CheckLockdown ()
+		{
+			GConf.Client client = new GConf.Client ();
+
+			try {
+				disable_command_line = (bool) client.Get ("/desktop/gnome/lockdown/disable_command_line");
+			} catch {
+				// The key isn't set for some reason
+				disable_command_line = false;
+			}
+
+			checked_gconf = true;
+		}
+
 		// invalid .desktop files get filtered out by Validate(), so they won't
 		// show up as Application tiles, but will show up as File tiles. But
 		// valid .desktop files marked to not show up in GNOME get eaten by
@@ -67,6 +86,17 @@ namespace Search.Tiles {
 			if (onlyshow != null && onlyshow.IndexOf ("GNOME") == -1)
 				return null;
 
+
+			if (!checked_gconf)
+				CheckLockdown();
+
+			if (disable_command_line) {
+				string[] categories = hit.GetProperties ("fixme:Categories");
+
+				if (categories != null && Array.IndexOf (categories, "TerminalEmulator") != -1)
+					return null;
+			}
+
 			return new Application (hit, query, ditem);
 		}
 	}
